@@ -10,9 +10,9 @@ import traceback
 import uuid
 import re
 import _thread
-import asyncio  # Requerido
+import asyncio
 from jaeger_client import Config
-from opentracing import Format
+from opentracing import Format, child_of
 from opentracing.ext import tags as ext_tags
 from abc import abstractmethod, ABC
 from .options import Options
@@ -27,11 +27,14 @@ from .types import TypesActions
 from .HttpResponse import HttpResponse
 import redis
 import hashlib
+from contextvars import ContextVar, copy_context
 
 if os.environ.get('KAFKA', None):
     from .ms_base import KafkaBase as BaseClass
 elif os.environ.get('NATS', None):
     from .NatsBase import NatsBase as BaseClass
+elif os.environ.get('REST_ENABLED', None):
+    from .HttpTransit import HttpTransit as BaseClass
 else:
     # En caso que se no indique por variable de entorno
     from .ms_base import KafkaBase as BaseClass
@@ -83,6 +86,9 @@ class Microservices(BaseClass, ABC):
     # Instancia de redis
     redisclient = None
 
+    # Contex Span de tracer
+    __SPAM = ContextVar('span')
+
     def __init__(self, opt):
         # Validar que se pase un objeto de configuracion correcto
         if not isinstance(opt, Options):
@@ -134,7 +140,11 @@ class Microservices(BaseClass, ABC):
         self.initRedis()
 
         # llamar el constructor padre
-        BaseClass.__init__(self, topico, opt.Hosts, topico)
+        # (self, topic, app_name, version, service_name):
+        if os.environ.get('REST_ENABLED', None):
+            BaseClass.__init__(self, topico, opt.App, opt.Version, opt.Name)
+        else:
+            BaseClass.__init__(self, topico, opt.Hosts, topico)
 
         if opt.isNats:
             # iniciar la aplicacion
@@ -151,12 +161,28 @@ class Microservices(BaseClass, ABC):
             finally:
                 self.loop.close()
 
+        if os.environ.get('REST_ENABLED', None):
+            self.__loop_async(self.start())
+
+    def __loop_async(self, fn):
+        """
+            Metodo para mandar llamar el ciclo del async
+        """
+        self.loop = asyncio.get_event_loop()
+        self.loop.run_until_complete(asyncio.wait([fn()]))
+        # self.loop.run_until_complete(self.run())
+        try:
+            self.loop.run_forever()
+        finally:
+            self.loop.close()
+
     def initRedis(self):
         """
             Metodo para inicializar el redis
         """
-        self.logs.info("Redis {}".format(
-            os.environ.get('REDISVALIDATE', None)))
+        if not os.environ.get('REDISVALIDATE', None) is None:
+            self.logs.info("Redis {}".format(
+                os.environ.get('REDISVALIDATE', None)))
 
         if os.environ.get('REDISVALIDATE', None) is None:
             self.logs.warn('No se encuentra activado la validacion por redis')
@@ -191,6 +217,7 @@ class Microservices(BaseClass, ABC):
             return True
 
     def initilizeTracer(self):
+        j = 'JAEGER_LOGS'
         config = Config(
             config={
                 'sampler': {
@@ -198,10 +225,16 @@ class Microservices(BaseClass, ABC):
                     'param': 1,
                 },
                 'local_agent': {
-                    'reporting_host': os.environ.get('JAEGER_AGENT_HOST', 'jaeger-agent.logging'),
-                    'reporting_port': os.environ.get('JAEGER_AGENT_PORT', 6831),
+                    'reporting_host': os.environ.get(
+                        'JAEGER_AGENT_HOST',
+                        'jaeger-agent.logging',
+                    ),
+                    'reporting_port': os.environ.get(
+                        'JAEGER_AGENT_PORT',
+                        6831,
+                    ),
                 },
-                'logging': False if os.environ.get('JAEGER_LOGS', None) is None else True,
+                'logging': False if os.environ.get(j, None) is None else True,
                 'tags': {
                     '@coppel-py-version': version
                 }
@@ -256,7 +289,7 @@ class Microservices(BaseClass, ABC):
             server = HTTPServer(('', 80), myHandler)
             server.serve_forever()
         else:
-            self.logs.warn('HttpHealthCheck is Disabled')
+            self.logs.warning('HttpHealthCheck is Disabled')
 
     def coro_manager(self, coro):
         """
@@ -305,16 +338,26 @@ class Microservices(BaseClass, ABC):
             Metodo para el envio de una peticion a un topico, sin esperar
             una respuesta del mismo.
         """
+        ctx_op = self.__SPAM.get(
+            {'span': None, "id_transaction": uuid.uuid4()})
+
+        span_ctx = ctx_op.get('span')
+
+        carrier = {}
 
         if topic == "Errores_criticos":
             # self._send(topic, data, id_transaction)
             self._send_error(data, id_transaction)
         else:
 
-            id_trans = id_transaction if id_transaction is None else str(
-                uuid.uuid4())
+            id_trans = id_transaction if id_transaction is not None else ctx_op.get(
+                "id_transaction")
 
-            self._send(topic, {
+            if span_ctx is not None:
+                self.___Tracer.inject(
+                    span_ctx, format=Format.TEXT_MAP, carrier=carrier)
+
+            DATA_SEND = {
                 "metadata": {
                     "uuid": None,
                     "id_operacion": str(uuid.uuid4()),
@@ -329,11 +372,11 @@ class Microservices(BaseClass, ABC):
                     "asynchronous": False,
                     "mtype": "WITHOUT_RESPONSE",
                     "bifurcacion": False,
-                    "time": "",
+                    "time": int(time.time() * 1000),
                     "from": socket.gethostname(),
                     "method": "GET" if action is None else action,
                     "deviceType": "",
-                    "span": None,
+                    "span": carrier if span_ctx is not None else None,
                     "filters": None,
                 },
                 "data": data,
@@ -341,7 +384,9 @@ class Microservices(BaseClass, ABC):
                     "Asynchronous": False,
                     "Authorizacion": ""
                 }
-            }, id_trans)
+            }
+
+            self._send(topic, DATA_SEND, id_trans)
 
     def _message(self, msg):
         """
@@ -376,12 +421,18 @@ class Microservices(BaseClass, ABC):
         if 'smoketest' in list(data):
             span = None
 
-            if rootSpan is None:
+            try:
+                if rootSpan is None:
+                    span = self.___Tracer.start_span(
+                        operation_name='smoketest')
+                else:
+                    span = self.___Tracer.start_span(
+                        operation_name='smoketest', child_of=rootSpan)
+            except Exception as e:
+                self.logs.warning(e)
                 span = self.___Tracer.start_span(operation_name='smoketest')
-            else:
-                span = self.___Tracer.start_span(
-                    operation_name='smoketest', child_of=rootSpan)
 
+            # Tag
             span.set_tag(ext_tags.HTTP_METHOD, msg['metadata']['method'])
 
             # Ejecutar la funcion del smoketest
@@ -400,17 +451,25 @@ class Microservices(BaseClass, ABC):
         if self.hasProcessMessage('{}_{}_{}'.format(
             self.__TOPIC__,
             meta.get('id_transaction', ''),
-            json.dumps(msg.get('data', {})
-                       )
+            json.dumps(
+                msg.get('data', {})
+            )
         )):
             span = None
 
+            spr = None
+
             if rootSpan is None:
                 span = self.___Tracer.start_span(
                     operation_name='replication')
             else:
+                if isinstance(rootSpan, dict):
+                    spr = self.___Tracer.extract(Format.TEXT_MAP, rootSpan)
+
                 span = self.___Tracer.start_span(
-                    operation_name='replication', child_of=rootSpan)
+                    operation_name='replication',
+                    references=child_of(spr if spr is not None else rootSpan)
+                )
 
             if os.environ.get('DISABLED_LOGS_VALIDATE', None) is None:
                 self.logs.error('Se recibio una operacion que ya se realizo')
@@ -508,11 +567,12 @@ class Microservices(BaseClass, ABC):
         except Exception as e:
             self.logs.error(e)
 
+        # Validar si es un string o un diccionario
         if rootSpan is not None:
             rootSpan = self.___Tracer.extract(Format.TEXT_MAP, rootSpan)
 
         span = self.___Tracer.start_span(operation_name=getattr(
-            mth, '__FUNC_NAME__'), child_of=rootSpan)
+            mth, '__FUNC_NAME__'), references=child_of(rootSpan))
 
         span.set_tag(ext_tags.HTTP_METHOD, msg['metadata']['method'])
 
@@ -530,8 +590,17 @@ class Microservices(BaseClass, ABC):
             # Indicar si es una bifurcacion
             msg['metadata']['bifurcacion'] = True if confForks else False
 
+            # Almacenar el valor de span en el contexto
+            self.__SPAM.set({
+                "span": span,
+                "id_transaction": meta.get('id_transaction', uuid.uuid4())
+            })
+
+            # Copiar el contexto de la ejucion
+            ctx = copy_context()
+
             # ejecutar el proceso
-            RESP = self.__APP.process(span, msg, mth, paramsConf)
+            RESP = ctx.run(self.__APP.process, span, msg, mth, paramsConf)
 
             # Ruta [SOLO NATST]
             id_route_nats = getItemBy('response.uuid', RESP)
@@ -642,6 +711,9 @@ class Microservices(BaseClass, ABC):
         # Topico de respuesta
         TOPIC_RESP = None
 
+        # Version del topico de respuesta
+        VERSION_RESP = None
+
         # Codigo de error
         errorCodeBk = 0
 
@@ -748,6 +820,7 @@ class Microservices(BaseClass, ABC):
                     dataResponse['metadata']['callbacks'].append({
                         "method": 'ENDPOINT',
                         "name": nats_route,
+                        "version": None,
                     })
 
             # Recuperar los callbacks
@@ -767,12 +840,14 @@ class Microservices(BaseClass, ABC):
                     dataResponse['metadata']['callbacks'].append({
                         "method": original_method,
                         "name": self.__TOPIC__,
+                        "version": self.__OPT.Version,
                     })
             else:
                 # Agregar al arreglo
                 dataResponse['metadata']['callbacks'].append({
                     "method": original_method,
                     "name": self.__TOPIC__,
+                    "version": self.__OPT.Version,
                 })
 
             # TOPIC_RESP = dataResponse['metadata']['worker']
@@ -790,12 +865,14 @@ class Microservices(BaseClass, ABC):
                         cllbs = callbacks.pop()
                         if self.__TOPIC__ != cllbs.get('name') or mthd != cllbs.get('method'):
                             TOPIC_RESP = cllbs.get('name')
+                            VERSION_RESP = cllbs.get('version', None)
                             dataResponse['metadata']['method'] = cllbs.get(
                                 'method')
                             find = False
                         counter += 1
                 else:
                     TOPIC_RESP = cllbs.get('name')
+                    VERSION_RESP = cllbs.get('version', None)
                     dataResponse['metadata']['method'] = cllbs.get('method')
 
             else:
@@ -826,6 +903,12 @@ class Microservices(BaseClass, ABC):
                     "[a-zA-Z0-9]{1,}[_.]{1}[a-zA-Z0-9]{1,}[_.]{1}.*",
                     TOPIC_RESP
                 )
+
+                # Validacion para la version 1 de los servicios legados
+                if not is_topic:
+                    if VERSION_RESP is not None:
+                        is_topic = True
+
                 # Asignar el id de ruta que proviene del reply
                 TOPIC_RESP = TOPIC_RESP if is_topic else nats_route
 
@@ -1033,7 +1116,7 @@ class Microservices(BaseClass, ABC):
         if hasattr(self, name):
             return getattr(self, name)
 
-    @abstractmethod
+    @ abstractmethod
     def smoketest(self):
         """
             Metodo que es llamado para validar los
